#define RFM69_TIMEOUTVAL    8000000UL

#include "global.h"



#define DEBUG


// Turn Receiver on and off
void switchToRecive(void) {
	sendSingleMode(0x01,0x10, 1); // RX on (set to receiver mode in RegOpMode)
	while (!(sendSingleMode(0x27,0x00, 0) & (1 << 7))); // Wait for Mode-Ready--Flag
}

void switchToTransmit(void) {
	sendSingleMode(0x01,0x0C, 1); // RX on (set to receiver mode in RegOpMode)
	while (!(sendSingleMode(0x27,0x00, 0) & (1 << 7))); // Wait for Mode-Ready--Flag
}

void switchToStandby(void) {
	sendSingleMode(0x01,0x04, 1); // RX on (set to receiver mode in RegOpMode)
	while (!(sendSingleMode(0x27,0x00, 0) & (1 << 7))); // Wait for Mode-Ready--Flag

}

void recive(char* data){

  #ifdef DEBUG
    uartSendString("\r\n");
    uartSendString("RECIVE vor standby : ");
    print_hexXX(sendSingleMode(0x27,0xFF, 0));
    print_hexXX(sendSingleMode(0x28,0xFF, 0));
  #endif


  switchToStandby(); // Turn off receiver, switch to Standby

  #ifdef DEBUG
    uartSendString("\r\n");
    uartSendString("RECIVE nacm standby : ");
    print_hexXX(sendSingleMode(0x27,0xFF, 0));
    print_hexXX(sendSingleMode(0x28,0xFF, 0));
  #endif

   // Read FIFO-data into FIFO-array
    enableSpi();
    spiTransferData(0x00); // Read-access register 0

	for (uint8_t i = 0; i < 9; i++) {
		data[i] = spiTransferData(0x00);
	}
	disableSpi();
	data[9] = '\0'; // Terminate string

	#ifdef DEBUG
    uartSendString("\r\n");
    uartSendString("RECIVE nach FIFO : ");
    print_hexXX(sendSingleMode(0x27,0xFF, 0));
    print_hexXX(sendSingleMode(0x28,0xFF, 0));
  #endif

}

void transmit(char* data){

	 switchToStandby(); // Turn off receiver, switch to Standby

   // Read FIFO-data into FIFO-array
  enableSpi();
  spiTransferData(0x80); // Write-access register 0

	for (uint8_t i = 0; i < 10; i++) {
		spiTransferData(data[i]);
	}

	disableSpi();

	#ifdef DEBUG
    uartSendString("\r\n");
    uartSendString("TRANSMIT Nach Schreiben FIFO: ");
    print_hexXX(sendSingleMode(0x27,0xFF, 0));
    print_hexXX(sendSingleMode(0x28,0xFF, 0));
  #endif


	// Turn on transmitter (Transmitting starts automatically if FIFO not empty)
	switchToTransmit();

	//	for(uint8_t i = 0 ; i <50 ; i++){
  //	print_hexXX(sendSingleMode(0x28,0xFF, 0));
  //	uartSendString("\r\n");
  //	}

  // Wait for Package Sent
	while (!(sendSingleMode(0x28,0xFF, 0) & (1 << 3))){
    #ifdef DEBUG
      uartSendString("\r\n");
      uartSendString("Wait: ");
      print_hexXX(sendSingleMode(0x27,0xFF, 0));
      print_hexXX(sendSingleMode(0x28,0xFF, 0));
    #endif
	}

	#ifdef DEBUG
    uartSendString("\r\n");
    uartSendString("TRANSMIT Nach WAIT-Schleife: ");
    print_hexXX(sendSingleMode(0x27,0xFF, 0));
    print_hexXX(sendSingleMode(0x28,0xFF, 0));
  #endif

	switchToStandby();
}

char sendSingleMode(char byte1, char byte2, char write){
  char result;

  if (write == 1){
    byte1 = byte1 | 0x80;
  }
  else{
    byte1 = byte1 & 0x7F;
    byte2 = 0x00;
  }

  enableSpi();
  spiTransferData(byte1);
  result = spiTransferData(byte2);
  disableSpi();
  return result;
}

void initRfm69(void){


  sendSingleMode(0x02,0x00, 1); // FSK, Packet mode, No shaping


  //Bitrate 4800kb/S
  sendSingleMode(0x03,0x1A, 1);
  sendSingleMode(0x04,0x0B, 1);

  //frequency deviation
  sendSingleMode(0x05,0x00, 1);
  sendSingleMode(0x06,0x52, 1);


  //Frequency = 433 MHZ --> 433000000 / 61 (FSTEP)
  sendSingleMode(0x07,0x6C, 1);
  sendSingleMode(0x08,0x40, 1);
  sendSingleMode(0x09,0x00, 1);


  // Set Output Power 8db --> 10001000
  sendSingleMode(0x11,0x48, 1);

  // Over current protection enabled, 100mA
  sendSingleMode(0x13,0x1B, 1);

  //LNA impedance
  sendSingleMode(0x18,0x00, 1); // LNA: 50 Ohm Input Impedance, Automatic Gain Control


  // Auto frequency correction auto on and clear
  sendSingleMode(0x1E,0x2D, 1);


  // DIO-Mapping
  sendSingleMode(0x25,0x00, 1); // Clkout, FifoFull, FifoNotEmpty, FifoLevel, PacketSent/CrcOk
  sendSingleMode(0x26,0x07, 1); // Clock-Out off



  sendSingleMode(0x29,0x4C, 1); // RSSI mind. -98 dBm
  sendSingleMode(0x2A,0x00, 1); // No Timeout after Rx-Start if no RSSI-Interrupt occurs
  sendSingleMode(0x2B,0x28, 1); // Timeout after RSSI-Interrupt if no Payload-Ready-Interrupt occurs

  // Preamble length 4 bytes
  sendSingleMode(0x2C,0x00, 1);
  sendSingleMode(0x2D,0x38, 1);

  // Sync-Mode
  sendSingleMode(0x2E,0x98, 1); // set FIFO mode
  sendSingleMode(0x2F,0x11, 1); // sync word 1 byte
  sendSingleMode(0x30,0x12, 1); // sync word 2 byte
  sendSingleMode(0x31,0x13, 1); // sync word 3 byte
  sendSingleMode(0x32,0x14, 1); // sync word 4 byte
  sendSingleMode(0x33,0x15, 1); // sync word 5 byte
  sendSingleMode(0x34,0x16, 1); // sync word 6 byte
  sendSingleMode(0x35,0x17, 1); // sync word 7 byte
  sendSingleMode(0x36,0x18, 1); // sync word 8 byte

  // Packet config
  sendSingleMode(0x37,0x38, 1); // fix length, Manchester, CRC-Check, No CRCAutoClean , No Address filter
  sendSingleMode(0x38,0x0A, 1); // Max. Payload-Length
  sendSingleMode(0x3C,0x80, 1); // Tx-Start-Condition: FIFO not empty
  sendSingleMode(0x3D,0xA0, 1); // Packet-Config2


  // Receiver config
  //sendSingleMode(0x58,0x2D, 1); // High sensitivity mode
  //sendSingleMode(0x6F,0x30, 1); // Improved DAGC

	// Start RC-Oscillator
	sendSingleMode(0x0A,0x80, 1);

	// Wait for RC-Oscillator
  uint32_t  utimer = RFM69_TIMEOUTVAL;
	while (!(sendSingleMode(0x0A,0x00, 0) & (1 << 6)) && --utimer);

	switchToRecive();
}

uint8_t rfm_receiving(void) {


	// No Payload and RSSI-Rx-Timeout -> Rx-Restart
	if (!(sendSingleMode(0x27,0x00,0) & (1 << 2)) && (sendSingleMode(0x28,0x00,0) & (1 << 2))) {
		sendSingleMode( 0x3D,0xA4, 1);
	}

	// Check if PayloadReady is set AND unused bit is not set (if bit 0 is set, module is not plugged in)
	return ((sendSingleMode(0x28,0x00,0) & (1 << 2)) && !(sendSingleMode(0x28,0x00,0) & (1 << 0)));
}
